@@ -21,10 +21,8 @@
 
 #include "kerncompat.h"
 #include <stddef.h>
-#include "kernel-lib/bitops.h"
 #include "kernel-lib/sizes.h"
 #include "kernel-shared/ctree.h"
-#include "kernel-shared/uapi/btrfs.h"
 
 struct btrfs_tree_parent_check;
 struct btrfs_fs_devices;
@@ -128,19 +126,19 @@ enum btrfs_open_ctree_flags {
 enum btrfs_read_sb_flags {
 	SBREAD_DEFAULT		= 0,
 	/* Reading superblock during recovery */
-	SBREAD_RECOVER		= (1 << 0),
+	SBREAD_RECOVER		= (1U << 0),
 
 	/*
 	 * Read superblock with the fake signature, cannot be used with
 	 * SBREAD_RECOVER
 	 */
-	SBREAD_TEMPORARY = (1 << 1),
+	SBREAD_TEMPORARY	= (1U << 1),
 
 	/*
 	 * Equivalent of OPEN_CTREE_IGNORE_FSID_MISMATCH, allow to read
 	 * superblock that has mismatched sb::fsid and sb::dev_item.fsid
 	 */
-	SBREAD_IGNORE_FSID_MISMATCH = (1 << 2),
+	SBREAD_IGNORE_FSID_MISMATCH = (1U << 2),
 };
 
 /*
@@ -225,8 +223,7 @@ void btrfs_mark_buffer_dirty(struct extent_buffer *buf);
 int btrfs_buffer_uptodate(struct extent_buffer *buf, u64 parent_transid,
 			  int atomic);
 int btrfs_set_buffer_uptodate(struct extent_buffer *buf);
-int btrfs_csum_data(struct btrfs_fs_info *fs_info, u16 csum_type, const u8 *data,
-		    u8 *out, size_t len);
+int btrfs_csum_data(u16 csum_type, const u8 *data, u8 *out, size_t len);
 
 int btrfs_open_device(struct btrfs_device *dev);
 int csum_tree_block_size(struct extent_buffer *buf, u16 csum_sectorsize,
@@ -237,10 +234,11 @@ int btrfs_read_buffer(struct extent_buffer *buf, u64 parent_transid);
 int write_tree_block(struct btrfs_trans_handle *trans,
 		     struct btrfs_fs_info *fs_info,
 		     struct extent_buffer *eb);
-int btrfs_fs_roots_compare_roots(struct rb_node *node1, struct rb_node *node2);
+int btrfs_fs_roots_compare_roots(const struct rb_node *node1, const struct rb_node *node2);
 struct btrfs_root *btrfs_create_tree(struct btrfs_trans_handle *trans,
-				     struct btrfs_fs_info *fs_info,
 				     struct btrfs_key *key);
+int btrfs_clear_tree(struct btrfs_trans_handle *trans,
+		     struct btrfs_root *root);
 int btrfs_delete_and_free_root(struct btrfs_trans_handle *trans,
 			       struct btrfs_root *root);
 struct btrfs_root *btrfs_csum_root(struct btrfs_fs_info *fs_info, u64 bytenr);
